/* Tracker Extract - extracts embedded metadata from files
 * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
 * Flac support added by Michael Cronenworth (mike@cchtml.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#include "config.h"

#include <stdio.h>
#include <string.h>
#include <fcntl.h>

#include <glib.h>

#include <libmodplug/modplug.h>

#include "tracker-ripped-functions.h"
#include "tracker-extract-module-api.h"

static void extract_mod (const char *filename,
                          GHashTable *metadata);

static TrackerExtractData extract_data[] = {
	{ "audio/x-mod", extract_mod },
	{ "audio/x-s3m", extract_mod },
	{ "audio/x-xm", extract_mod },
	{ "audio/x-it", extract_mod },
	{ NULL, NULL }
};


static void
extract_mod (const char *filename, GHashTable *metadata)
{
gchar *title;
gchar *data;
gint len;
gsize dlen;

ModPlugFile *mod;

if (!g_file_get_contents(filename, &data, &dlen, NULL))
	return;

mod=ModPlug_Load(data, dlen);

if (!mod)
	return;

title=ModPlug_GetName(mod);
len=ModPlug_GetLength(mod);

g_hash_table_insert(metadata, g_strdup ("Audio:Codec"), g_strdup("modplug"));
g_hash_table_insert(metadata, g_strdup ("Audio:Channels"), g_strdup("2"));
g_hash_table_insert(metadata, g_strdup ("Audio:Samplerate"), g_strdup("44100"));
g_hash_table_insert(metadata, g_strdup ("Audio:Duration"), g_strdup_printf("%d", len/1000));

/* Modules can mostly only contain a title,
   the sample names have usually been used as artist info,
   but we can't parse that so fake something so they are nicely grouped.
 */
g_hash_table_insert(metadata, g_strdup ("Audio:Artist"), g_strdup("Various Artists"));
g_hash_table_insert(metadata, g_strdup ("Audio:Album"), g_strdup("Tracker Modules"));
if (title)
	g_hash_table_insert(metadata, g_strdup ("Audio:Title"), g_strdup(title));

ModPlug_Unload(mod);
g_free(data);
}

TrackerExtractData *
tracker_get_extract_data (void)
{
return extract_data;
}
